@@ -96,12 +96,6 @@ class StaticFn_1To1(BlockPrototype):
 
         return self.outputSignal(0)
 
-    def codeGen_localvar(self, language, signal):
-        # TODO: every block prototype shall befine its variables like this.. move this to BlockPrototype and remove all individual implementations
-
-        if language == 'c++':
-            return cgh.defineVariableLine( signal )
-
 
 
 
@@ -150,11 +144,6 @@ class StaticFn_NTo1(BlockPrototype):
 
         return output
 
-    def codeGen_localvar(self, language, signal):
-        if language == 'c++':
-            return cgh.defineVariableLine( signal )
-
-
 
 class Dynamic_1To1(BlockPrototype):
     def __init__(self, sim : Simulation, u : Signal ):
@@ -190,10 +179,6 @@ class Dynamic_1To1(BlockPrototype):
 
         return self.outputSignal(0)
 
-    def codeGen_localvar(self, language, signal):
-        if language == 'c++':
-            return cgh.defineVariableLine( signal )
-
 
 
 
@@ -444,12 +429,7 @@ class GenericSubsystem(BlockPrototype):
     def codegen_addToNamespace(self, language):
         lines = ''
 
-        if self.compileResult is not None:
-            # add the code of the subsystem
-            lines = self.compileResult.commandToExecute.codeGen(language, 'code')
-
-            if lines is None:
-                raise BaseException("lines is None")
+        # putting code for subsystems is performed using execution commands
 
         return lines
 
@@ -464,26 +444,20 @@ class GenericSubsystem(BlockPrototype):
         if language == 'c++':
             return self.instanceVarname + '.' + self.manifest.getAPIFunctionName('reset') +  '();\n'
 
-    def codeGen_localvar(self, language, signal):
-        if language == 'c++':
-            self.isSignalVariableDefined[ signal ] = True
-
-            return cgh.defineVariableLine( signal )
 
     def codeGen_init(self, language):
+        pass
 
-        # isSignalVariableDefined contains for each output signal a flag that indicates wheter veriables 
-        # for these output signals shall be defined (i.e., memory reserved)
-        self.isSignalVariableDefined = {}
-        for s in self.outputSignals:
-            self.isSignalVariableDefined[ s ] = False
 
     def codeGen_destruct(self, language):
         pass
 
+
+    # helper fn to build code
     def codeGen_call_OutputFunction(self, instanceVarname, manifest, language):
         return instanceVarname + '.' + manifest.getAPIFunctionName('calculate_output') +  '(' + cgh.signalListHelper_names_string(self.outputSignals + self.inputsToCalculateOutputs) + ');\n'
 
+    # helper fn to build code
     def codeGen_call_UpdateFunction(self, instanceVarname, manifest, language):
         return instanceVarname + '.' + manifest.getAPIFunctionName('state_update') +  '(' + cgh.signalListHelper_names_string(self.inputsToUpdateStates) + ');\n'
 
@@ -492,19 +466,17 @@ class GenericSubsystem(BlockPrototype):
         if language == 'c++':
             lines = ''
             
-            for signal, isDefined in self.isSignalVariableDefined.items():
-                # if the signal is not a simulation output
+            #
+            # TODO: 2.5.2020: concept: how to compute only the nescessary signals?
+            #
 
-                if not isDefined:
+            for s in self.outputSignals:
+                lines += cgh.defineVariable( s ) 
 
-                    lines += cgh.defineVariable( signal ) 
-
-                    if signal not in signals:
-                        lines += ' // NOTE: unused output signal\n'
-                    else:
-                        lines += '\n'
-
-                    self.isSignalVariableDefined[ signal ] = True
+                if s not in signals:
+                    lines += '// NOTE: unused output signal' + s.name + '\n'
+                else:
+                    lines += ''                
 
             lines += self.codeGen_call_OutputFunction(self.instanceVarname, self.manifest, language)
 
@@ -678,13 +650,12 @@ class MultiSubsystemEmbedder(BlockPrototype):
 
         lines = '{ // subsystem ' + system_prototype.embedded_subsystem.name + '\n'
 
+        innerLines = ''
 
         # generate code for calculating the outputs 
         if calculate_outputs:
-            innerLines = system_prototype.codeGen_output_list(language, system_prototype.outputs)
 
-            #if self._number_of_switched_outputs != len( ouput_signals_name ):
-            #    raise BaseException(".")
+            innerLines += system_prototype.codeGen_output_list(language, system_prototype.outputs)
 
             for i in range( 0, len( ouput_signals_name ) ):
                 innerLines += cgh.asign( system_prototype.outputs[i].name, ouput_signals_name[i] )
@@ -697,7 +668,7 @@ class MultiSubsystemEmbedder(BlockPrototype):
 
         # generate code for updating the states
         if update_states:
-            innerLines = system_prototype.codeGen_update(language)
+            innerLines += system_prototype.codeGen_update(language)
 
 
         lines += cgh.indent(innerLines)
@@ -844,11 +815,19 @@ class StatemachineSwichSubsystems(MultiSubsystemEmbedder):
         # how to add more outputs?
         self.state_output.setDatatype( DataTypeInt32(1) )
 
+        # this output signals must be compted in any way
+        # also in case it is not used by other blocks
+        sim.add_signal_mandatory_to_compute( self.state_output )
+
 
     @property
     def state_output(self):
         return self.additional_outputs[0]
 
+    # def returnInutsToUpdateStates(self, outputSignal):
+ 
+    #     return MultiSubsystemEmbedder.returnInutsToUpdateStates(self, outputSignal) + [ self.state_output ]
+
 
     def codeGen_reset(self, language):
         if language == 'c++':
@@ -873,10 +852,18 @@ class StatemachineSwichSubsystems(MultiSubsystemEmbedder):
 
         lines = ''
         if language == 'c++':
+
+            # do not compute all swich outputs, just the ones requested by 'signals' 
+            outputs_to_compute = []
+            for s in signals:
+                if s in self.subsystem_switch_outouts:
+                    outputs_to_compute.append( s )
+
+
             # lines += cgh.defineVariables( signals ) + '\n'
             lines += self.generate_switch( language=language, 
                                             switch_control_signal_name=  self._state_memory ,
-                                            switch_ouput_signals_name=cgh.signalListHelper_names(self.subsystem_switch_outouts),
+                                            switch_ouput_signals_name=cgh.signalListHelper_names(outputs_to_compute),
                                             additional_outputs_names=[self.state_output.name] )
 
             # lines += self.state_output.name + ' = ' + self._state_memory_delayed + ';\n'
@@ -979,13 +966,6 @@ class TruggeredSubsystem(GenericSubsystem):
         return dependingInputsOuter
         
 
-
-    def codeGen_localvar(self, language, signal):
-        if language == 'c++':
-
-            return GenericSubsystem.codeGen_localvar(self, language, signal)
-
-
     def codeGen_call_OutputFunction(self, instanceVarname, manifest, language):
 
         if self.prevent_output_computation:
@@ -1043,13 +1023,6 @@ class ForLoopSubsystem(GenericSubsystem):
         return dependingInputsOuter
         
 
-
-    def codeGen_localvar(self, language, signal):
-        if language == 'c++':
-
-            return GenericSubsystem.codeGen_localvar(self, language, signal)
-
-
     def codeGen_call_OutputFunction(self, instanceVarname, manifest, language):
         lines = 'for (int i = 0; i < '  +  self.i_max_signal.name + '; ++i) {\n'
         lines += GenericSubsystem.codeGen_call_OutputFunction(self, instanceVarname, manifest, language)
@@ -1088,10 +1061,6 @@ class Const(StaticSource_To1):
         # call super
         StaticSource_To1.__init__(self, sim, datatype)
 
-    def codeGen_localvar(self, language, signal):
-        if language == 'c++':
-            return cgh.defineVariableLine( signal )
-
     def codeGen_output_list(self, language, signals : List [ Signal ] ):
         if language == 'c++':
             return signals[0].name + ' = ' + str( self.constant ) + ';\n'
