@@ -14,15 +14,24 @@ class PutRuntimeCppHelper:
         generates code for the runtime evironment
     """
 
-    def __init__(self, compileResults : CompileResults ):
+    def __init__(self, compile_results : CompileResults = None ):
         ExecutionCommand.__init__(self)  # TODO: what is this?
 
-        self.compileResults = compileResults
-        self.mainSimulation = compileResults.commandToExecute
+        if compile_results is not None:
+            self.compileResults = compile_results
+            self.mainSimulation = compile_results.commandToExecute
+
+        else:
+            self.compileResults = None
+            self.mainSimulation = None
 
         # list of inlcuded system
         self._includedSystems = []
 
+    def set_compile_results(self, compile_results : CompileResults ):
+        self.compileResults = compile_results
+        self.mainSimulation = compile_results.commandToExecute
+
     def include_systems(self, system : SystemLibraryEntry):
         self._includedSystems = system
 
@@ -47,7 +56,7 @@ class PutRuntimeCppHelper:
         # TODO: iterate over all functions present in the API of the system
         # NOTE: Currently only the main functions are used: output, update, and reset
         #
-        API_functions = self.mainSimulation.API_functions
+        API_functions = self.mainSimulation.command_to_put_main_system.API_functions
 
         #
         # make strings 
@@ -63,19 +72,19 @@ class PutRuntimeCppHelper:
 
         # for the output signals
         # input1_NamesCSVList; list of output signals. e.g. 'y1, y2, y3' 
-        outputNamesCSVList, outputNamesVarDef, outputPrinfPattern = makeStrings( self.mainSimulation.outputCommand.outputSignals )
+        outputNamesCSVList, outputNamesVarDef, outputPrinfPattern = makeStrings( self.mainSimulation.command_to_put_main_system.outputCommand.outputSignals )
 
         # the inputs to the output command
         # input1_NamesCSVList: list of output signals. e.g. 'y1, y2, y3' 
-        input1_NamesCSVList, input1_NamesVarDef, input1PrinfPattern = makeStrings( self.mainSimulation.outputCommand.inputSignals )
+        input1_NamesCSVList, input1_NamesVarDef, input1PrinfPattern = makeStrings( self.mainSimulation.command_to_put_main_system.outputCommand.inputSignals )
 
         # the inputs to the update command
         # input2_NamesCSVList list of output signals. e.g. 'u1, u2, u3' 
-        input2_NamesCSVList, input2_NamesVarDef, input2_PrinfPattern = makeStrings( self.mainSimulation.updateCommand.inputSignals )
+        input2_NamesCSVList, input2_NamesVarDef, input2_PrinfPattern = makeStrings( self.mainSimulation.command_to_put_main_system.updateCommand.inputSignals )
 
         # all inputs
         # merge the list of inputs for the calcoutput and stateupdate function
-        allInputs = list(set(self.mainSimulation.outputCommand.inputSignals + self.mainSimulation.updateCommand.inputSignals))
+        allInputs = list(set(self.mainSimulation.command_to_put_main_system.outputCommand.inputSignals + self.mainSimulation.command_to_put_main_system.updateCommand.inputSignals))
         inputAll_NamesCSVList, inputAll_NamesVarDef, inputAll_PrinfPattern = makeStrings( allInputs )
 
         # the names of input and output signals of the outputCommand combined
@@ -86,11 +95,11 @@ class PutRuntimeCppHelper:
         # calcOutputsArgsList.extend(  signalListHelper_names( self.mainSimulation.outputCommand.inputSignals ) )
         # calcOutputsArgs = ', '.join( calcOutputsArgsList )
 
-        calcOutputsArgs = cgh.signalListHelper_names( self.mainSimulation.outputCommand.outputSignals + self.mainSimulation.outputCommand.inputSignals )
+        calcOutputsArgs = cgh.signalListHelper_names( self.mainSimulation.command_to_put_main_system.outputCommand.outputSignals + self.mainSimulation.command_to_put_main_system.outputCommand.inputSignals )
 
         # fill in template
         self.template = Template(self.template).safe_substitute(  
-                                                    mainSimulationName = self.mainSimulation.getAPI_name(),
+                                                    mainSimulationName = self.mainSimulation.command_to_put_main_system.API_name,
                                                     simulationCode=simulationCode,
 
                                                     input1_NamesVarDef=input1_NamesVarDef,
@@ -113,8 +122,6 @@ class PutRuntimeCppHelper:
 
     def writeFiles(self, folder):
 
-        print(folder)
-
         with open( os.path.join( folder + '//simulation_manifest.json' ), 'w') as outfile:  
             json.dump(self.manifest, outfile)
 
@@ -132,12 +139,11 @@ class PutBasicRuntimeCpp(PutRuntimeCppHelper):
         generates code for the runtime evironment
     """
 
-    def __init__(self, compileResults : CompileResults, input_signals_mapping ):
+    def __init__(self, input_signals_mapping = {} ):
 
-        PutRuntimeCppHelper.__init__(self, compileResults)
+        PutRuntimeCppHelper.__init__(self)
 
         self.input_signals_mapping = input_signals_mapping
-
         self.initCodeTemplate()
 
         
@@ -183,7 +189,7 @@ class PutBasicRuntimeCpp(PutRuntimeCppHelper):
 
         # parse csv data
         data = [ ]
-        outputs = range(0, len(self.mainSimulation.outputCommand.outputSignals) )
+        outputs = range(0, len(self.mainSimulation.command_to_put_main_system.outputCommand.outputSignals) )
 
         for o in outputs:
             data.append( [] )
@@ -197,7 +203,7 @@ class PutBasicRuntimeCpp(PutRuntimeCppHelper):
         # put data into a key-array
         dataStruct = { }
         o = 0
-        for s in self.mainSimulation.outputCommand.outputSignals:
+        for s in self.mainSimulation.command_to_put_main_system.outputCommand.outputSignals:
             dataStruct[ s.name ] = data[o]
 
             o = o + 1
@@ -265,20 +271,19 @@ int main () {
 
 
 
-class WasmRuntimeCpp(PutRuntimeCppHelper):
+class WasmRuntime(PutRuntimeCppHelper):
     """
-        generates code for the runtime evironment
+        generates code for the Webassemble runtime evironment
 
         https://emscripten.org/docs/porting/connecting_cpp_and_javascript/embind.html
 
     """
 
-    def __init__(self, compileResults : CompileResults, input_signals_mapping ):
+    def __init__(self, input_signals_mapping = {} ):
 
-        PutRuntimeCppHelper.__init__(self, compileResults)
+        PutRuntimeCppHelper.__init__(self)
 
         self.input_signals_mapping = input_signals_mapping
-
         self.initCodeTemplate()
 
         
@@ -298,10 +303,10 @@ class WasmRuntimeCpp(PutRuntimeCppHelper):
 
 
         # build I/O structs
-        ioExport = self.codeGen_writeIO(self.mainSimulation.outputCommand)
-        ioExport += self.codeGen_writeIO(self.mainSimulation.updateCommand)
+        ioExport = self.codeGen_writeIO(self.mainSimulation.command_to_put_main_system.outputCommand)
+        ioExport += self.codeGen_writeIO(self.mainSimulation.command_to_put_main_system.updateCommand)
 
-        ioExport += self.codeGen_writeIO(self.mainSimulation.resetCommand)
+        ioExport += self.codeGen_writeIO(self.mainSimulation.command_to_put_main_system.resetCommand)
 
         self.template = Template(self.template).safe_substitute( ioExport=ioExport,
                                                                  inputConstAssignment=inputConstAssignment    ) 
@@ -324,7 +329,7 @@ class WasmRuntimeCpp(PutRuntimeCppHelper):
             structPrefix = 'Outputs_'
             signals = command_API.outputSignals
 
-        mainSimulationName = self.mainSimulation.getAPI_name()
+        mainSimulationName = self.mainSimulation.command_to_put_main_system.API_name
 
         lines = ''
 
@@ -359,7 +364,7 @@ class WasmRuntimeCpp(PutRuntimeCppHelper):
 
     def build(self):
 
-        buildCommand = "emcc --bind " + os.path.join(self.codeFolder + "main.cpp") + " -g4 -s -o " + os.path.join( self.codeFolder + "main.js" )
+        buildCommand = 'emcc --bind -s MODULARIZE=1 -s EXPORT_NAME="ORTD_simulator" '  + os.path.join(self.codeFolder + "main.cpp") + " -g4 -s -o " + os.path.join( self.codeFolder + "main.js" )
         print("Running compiler: " + buildCommand)
 
         returnCode = os.system(buildCommand)
