@@ -48,6 +48,7 @@ class ExecutionCommand(object):
         return lines
 
 
+# rename to CommandCalculateSignalValues
 class CommandCalculateOutputs(ExecutionCommand):
     """
         execute an executionLine i.e. call the output-flags of all blocks given in executionLine
@@ -55,14 +56,18 @@ class CommandCalculateOutputs(ExecutionCommand):
         in executionLine.getSignalsToExecute()
 
         system - the system of which to calculate the outputs
+        target_signals - the signals to evaluate
+        output_signal - signals foreseen to be system outputs (e.g. for them no memory needs to be allocated)
     """
 
-    def __init__(self, system, executionLine, targetSignals, no_memory_for_output_variables : bool):
+    def __init__(self, system, executionLine, targetSignals, signals_from_system_states = [], no_memory_for_output_variables : bool = False, output_signals = []):
         ExecutionCommand.__init__(self)
 
-        self._system = system
-        self.executionLine = executionLine
-        self.targetSignals = targetSignals
+        self._system                          = system
+        self.executionLine                    = executionLine
+        self.targetSignals                    = targetSignals
+        self._output_signals                  = output_signals
+        self._signals_from_system_states      = signals_from_system_states
         self.define_variables_for_the_outputs = not no_memory_for_output_variables
 
     def printExecution(self):
@@ -92,37 +97,47 @@ class CommandCalculateOutputs(ExecutionCommand):
             if flag == 'localvar':
 
                 # 
-                SignalsWithoutOutputs = self.executionLine.getSignalsToExecute().copy()
+                signals_reduced_set = self.executionLine.getSignalsToExecute().copy()
 
                 # remove the system-output signals if requested
                 if not self.define_variables_for_the_outputs: # This is flipped by its name
                     for s in self.targetSignals:
-                        # s is a system output: the code that generates the source to calculate s shall not reserve memeory for s
 
-                        SignalsWithoutOutputs.remove( s )
+                        # if s is output signal of system 
+                        if s in self._output_signals:
 
-                        # notify the block prototype that the signal s will be a system output
-                        # and, hence, no memory shall be allocated for s (because the memory is already
-                        # available)
-                        s.getSourceBlock().getBlockPrototype().codeGen_setOutputReference('c++', s)
+                            # s is a system output: the code that generates the source to calculate s shall not reserve memeory for s
+
+                            signals_reduced_set.remove( s )
+
+                            # notify the block prototype that the signal s will be a system output
+                            # and, hence, no memory shall be allocated for s (because the memory is already
+                            # available)
+                            s.getSourceBlock().getBlockPrototype().codeGen_setOutputReference('c++', s)
+                            
+                            
+                            
 
 
                 # skip the input signals in this loop (as their variables are already defined by the function API)
-                for s in SignalsWithoutOutputs:
+                for s in signals_reduced_set:
 
-                    #if isinstance(s, BlockOutputSignal):
-                    if not s.is_crossing_system_boundary(self._system):
-                        # only implement caching for intermediate computaion results.
-                        # I.e. exclude the simulation input signals
+                    # remove also the variables that are states (e.g. in case they are defined by CommandRestoreCache(self._signals_from_system_states)
+                    if not s in self._signals_from_system_states:
 
-                        print('create local variable for signal ' + s.name + ' / ' + s.toStr() )
-                        lines += s.getSourceBlock().getBlockPrototype().codeGen_localvar('c++', s)
+                        if not s.is_crossing_system_boundary(self._system): # TODO: Why is this needed?
+                            # only implement caching for intermediate computaion results.
+                            # I.e. exclude the simulation input signals
+
+                            # print('create local variable for signal ' + s.name + ' / ' + s.toStr() )
+
+                            lines += cgh.defineVariableLine( s )
 
 
 
             if flag == 'code':
                 lines += '\n// calculating the block outputs in the following order ' + cgh.signalListHelper_names_string(self.executionLine.signalOrder ) + '\n'
-                lines += '// that depend on ' + cgh.signalListHelper_names_string(self.executionLine.dependencySignals) + '\n'
+                lines += '// that depend on ' + cgh.signalListHelper_names_string(self.executionLine.dependencySignalsSimulationInputs) + '\n'
                 lines += '// dependencies that require a state update are ' + cgh.signalListHelper_names_string(self.executionLine.dependencySignalsThroughStates) + ' \n'
                 lines += '\n'
 
@@ -144,6 +159,8 @@ class CommandCalculateOutputs(ExecutionCommand):
                             blocks_with_outputs_to_compute[ block ].append( s )
 
 
+                # for each blocks that provides outputs that are needed to compute,
+                # generate the code to calculate these outputs.
                 for block in blocks_with_outputs_to_compute:
                     lines += block.getBlockPrototype().codeGen_output_list('c++', blocks_with_outputs_to_compute[ block ] )
                     
@@ -248,10 +265,6 @@ class CommandUpdateStates(ExecutionCommand):
                 for b in self.blockList:
                     lines += b.getBlockPrototype().codeGen_update('c++')
 
-            # if flag == 'codereset':
-            #     lines += ''
-            #     for b in self.blockList:
-            #         lines += b.getBlockPrototype().codeGen('c++', 'reset')
 
         return lines
 
@@ -266,7 +279,6 @@ class CommandCacheOutputs(ExecutionCommand):
         copy the value of each given signal to the space of global variables
         (only signals that are the output of a block are considered, i.e. no 
         simulation inputs)
-
     """
 
     def __init__(self, signals : List[Signal]):
@@ -299,7 +311,7 @@ class CommandCacheOutputs(ExecutionCommand):
 
             if flag == 'variables':
                 lines += ''
-                lines += "\n\n// cached output values\n"
+                lines += "\n\n//\n// cached output values\n//\n"
                 for s in self.signals:
 
                     cachevarName = s.name + "__" + s.getSourceBlock().getBlockPrototype().getUniqueVarnamePrefix()
@@ -385,13 +397,14 @@ class PutAPIFunction(ExecutionCommand):
     # Creates an API-function to return the calculated values that might depend on input values
     # 
 
-    def __init__(self, nameAPI : str, inputSignals : List[ Signal ], outputSignals : List[ Signal ], executionCommands):
+    def __init__(self, nameAPI : str, inputSignals : List[ Signal ], outputSignals : List[ Signal ], executionCommands, generate_wrappper_functions = True):
         ExecutionCommand.__init__(self)
 
         self.outputSignals = outputSignals
         self.inputSignals = inputSignals
         self.executionCommands = executionCommands
         self._nameAPI = nameAPI
+        self._generate_wrappper_functions = generate_wrappper_functions
 
         for e in executionCommands:
             e.setContext(self)
@@ -421,7 +434,7 @@ class PutAPIFunction(ExecutionCommand):
 
     def codeGen_destruct(self, language):
         for c in self.executionCommands:
-            c.codeGen_init(language)
+            c.codeGen_destruct(language)
 
     def codeGen(self, language, flag):
 
@@ -435,9 +448,14 @@ class PutAPIFunction(ExecutionCommand):
 
 
             if flag == 'code':
-                # define the API-function (start)
-                lines += '// calculate ' + ' '.join( cgh.signalListHelper_names( self.outputSignals ) )
-                
+                #
+                # ------ define the API-function ------
+                #
+                if len(self.outputSignals) > 0:
+                    lines += '// API-function ' + self._nameAPI + ' to compute: ' + cgh.signalListHelper_names_string( self.outputSignals )  #  ' '.join( cgh.signalListHelper_names( self.outputSignals ) )
+                else:
+                    lines += '// API-function' + self._nameAPI
+
                 lines += '\n'
                 lines += 'void ' + self._nameAPI + '('
 
@@ -469,62 +487,68 @@ class PutAPIFunction(ExecutionCommand):
                 # define the API-function (finish)
                 lines += '}\n\n'
 
-                # put structs to hold I/O signals
-                lines += '// output data structure for ' + self._nameAPI + '\n'
-                tmp = cgh.defineVariables( self.outputSignals )
-                tmp = indent(tmp, '  ')
-                lines += f'struct Outputs_{ self._nameAPI }  {{\n{ tmp }}};\n\n'
-
-                lines += '// input data structure for ' + self._nameAPI + '\n'
-                tmp = cgh.defineVariables( self.inputSignals )
-                tmp = indent(tmp, '  ')
-                lines += f'struct Inputs_{ self._nameAPI }  {{\n{ tmp }}};\n\n'
-
-
                 #
-                # put a wrapper function that offers an API using structs for in- and output signals
+                # ------ end of 'define the API-function' ------
                 #
 
-                # put function header
-                lines += '// wrapper function for ' + self._nameAPI + '\n'
-                lines += 'Outputs_' + self._nameAPI + ' ' + self._nameAPI + '__ (Inputs_' + self._nameAPI + ' inputs)\n'
+                if self._generate_wrappper_functions:
 
-                if len(self.outputSignals) > 0 or len(self.inputSignals):
+                    # put structs to hold I/O signals
+                    lines += '// output data structure for ' + self._nameAPI + '\n'
+                    tmp = cgh.defineVariables( self.outputSignals )
+                    tmp = indent(tmp, '  ')
+                    lines += f'struct Outputs_{ self._nameAPI }  {{\n{ tmp }}};\n\n'
 
-                    outputArguments = cgh.getStructElements( 'outputs' , self.outputSignals )
-                    inputArguments = cgh.getStructElements( 'inputs' , self.inputSignals )
+                    lines += '// input data structure for ' + self._nameAPI + '\n'
+                    tmp = cgh.defineVariables( self.inputSignals )
+                    tmp = indent(tmp, '  ')
+                    lines += f'struct Inputs_{ self._nameAPI }  {{\n{ tmp }}};\n\n'
 
-                    argumentsString = ''
-                    if len(outputArguments) > 0:
-                        argumentsString += ', '.join( outputArguments )
 
-                    if len(outputArguments) > 0 and len(inputArguments) > 0:
-                        argumentsString += '   ,   '                    
+                    #
+                    # put a wrapper function that offers a 'nicer' API using structs for in- and output signals
+                    #
 
-                    if len(inputArguments) > 0:
-                        argumentsString += ', '.join( inputArguments )
+                    # put function header
+                    lines += '// wrapper function for ' + self._nameAPI + '\n'
+                    lines += 'Outputs_' + self._nameAPI + ' ' + self._nameAPI + '__ (Inputs_' + self._nameAPI + ' inputs)\n'
 
-                    print( 'args are: ' + argumentsString )
+                    if len(self.outputSignals) > 0 or len(self.inputSignals):
 
+                        outputArguments = cgh.getStructElements( 'outputs' , self.outputSignals )
+                        inputArguments = cgh.getStructElements( 'inputs' , self.inputSignals )
+
+                        argumentsString = ''
+                        if len(outputArguments) > 0:
+                            argumentsString += ', '.join( outputArguments )
+
+                        if len(outputArguments) > 0 and len(inputArguments) > 0:
+                            argumentsString += '   ,   '                    
+
+                        if len(inputArguments) > 0:
+                            argumentsString += ', '.join( inputArguments )
+
+                    else:
+                        argumentsString = ''
+
+                    lines += '{\n'
+
+                    innerLines = ''
+                    innerLines += cgh.defineStructVar( 'Outputs_' + self._nameAPI, 'outputs'  ) + '\n'
+
+                    innerLines += '// call to wrapped function\n'
+                    innerLines += self._nameAPI + '(' + argumentsString + ');\n'
+
+
+                    innerLines += '\n'
+                    innerLines += '// return the signals in a struct\n'
+                    innerLines += 'return outputs;\n'
+
+                    lines += indent(innerLines, '  ')
+                    
+                    lines += '}\n\n'
                 else:
-                    argumentsString = ''
-
-                lines += '{\n'
-
-                innerLines = ''
-                innerLines += cgh.defineStructVar( 'Outputs_' + self._nameAPI, 'outputs'  ) + '\n'
-
-                innerLines += '// call to wrapped function\n'
-                innerLines += self._nameAPI + '(' + argumentsString + ');\n'
-
-
-                innerLines += '\n'
-                innerLines += '// return the signals in a struct\n'
-                innerLines += 'return outputs;\n'
-
-                lines += indent(innerLines, '  ')
-                
-                lines += '}\n\n'
+                    print()
 
 
         return lines
@@ -535,13 +559,14 @@ class PutAPIFunction(ExecutionCommand):
 
 
 
-class PutSimulation(ExecutionCommand):
+class PutSystem(ExecutionCommand):
     """
         Represents a system that is represented by a class in c++
     """
 
-    def __init__(self, simulation : Simulation, resetCommand : ExecutionCommand, updateCommand : ExecutionCommand, outputCommand : ExecutionCommand ):
+    def __init__(self, system : Simulation, resetCommand : PutAPIFunction, updateCommand : PutAPIFunction, outputCommand : PutAPIFunction ):
         ExecutionCommand.__init__(self)
+        self.executionCommands = [ resetCommand, updateCommand, outputCommand  ] 
 
         self.resetCommand = resetCommand
         self.updateCommand = updateCommand
@@ -555,17 +580,20 @@ class PutSimulation(ExecutionCommand):
                          'state_update' : self.updateCommand,
                          'reset' : self.resetCommand }
 
-        self.executionCommands = [ resetCommand, updateCommand, outputCommand  ] 
 
-        self.simulation = simulation
-        self.nameAPI = simulation.getName()
+        self.system = system
+        self.nameAPI = system.getName()
 
         for e in self.executionCommands:
             e.setContext(self)
 
 
 
-    def getAPI_name(self):
+    # def getAPI_name(self):
+    #     return self.nameAPI
+
+    @property
+    def API_name(self):
         return self.nameAPI
 
     @property
@@ -578,7 +606,7 @@ class PutSimulation(ExecutionCommand):
 
     def printExecution(self):
 
-        print(Style.BRIGHT + Fore.YELLOW + "ExecutionCommand: Simulation with the API:")
+        print(Style.BRIGHT + Fore.YELLOW + "ExecutionCommand: Simulation with the API (" + self.nameAPI + "):")
         
         for c in self.executionCommands:
             c.printExecution()
@@ -586,16 +614,18 @@ class PutSimulation(ExecutionCommand):
         print(Style.BRIGHT + Fore.YELLOW + "}")
         
     def codeGen_init(self, language):
+
         for c in self.executionCommands:
             c.codeGen_init(language)
 
         # call init codegen for each block in the simulation
-        for block in self.simulation.blocks:
+        for block in self.system.blocks:
             block.getBlockPrototype().codeGen_init(language)
 
+
     def codeGen_destruct(self, language):
         for c in self.executionCommands:
-            c.codeGen_init(language)
+            c.codeGen_destruct(language)
 
     def codeGen(self, language, flag):
 
@@ -611,7 +641,7 @@ class PutSimulation(ExecutionCommand):
         
                 # Add code within the same namespace this simulation sits in.
                 # E.g. to add helper functions, classes, ...
-                for b in self.simulation.blocks:
+                for b in self.system.blocks:
                     lines += b.getBlockPrototype().codegen_addToNamespace(language)
 
                 # define the API-function (start)
@@ -630,7 +660,7 @@ class PutSimulation(ExecutionCommand):
                 for c in self.executionCommands:
                     innerLines += c.codeGen(language, 'code')
 
-                lines += indent(innerLines, '  ')
+                lines += cgh.indent(innerLines)
 
                 # define the API-function (finish)
                 lines += '};\n\n'
@@ -643,3 +673,74 @@ class PutSimulation(ExecutionCommand):
 
 
 
+class PutSystemAndSubsystems(ExecutionCommand):
+    """
+        Represents a system and its subsystem togehter that is represented by multiple classes in c++.
+        Addiitionally, they are packed into a namespace.
+    """
+
+    def __init__(self, command_to_put_main_system : PutSystem, commands_to_put_subsystems : PutSystem ):
+
+        ExecutionCommand.__init__(self)
+        self.executionCommands = commands_to_put_subsystems + [ command_to_put_main_system ] 
+
+        self._command_to_put_main_system = command_to_put_main_system
+        self._commands_to_put_subsystems = commands_to_put_subsystems
+
+    @property
+    def command_to_put_main_system(self):
+        return self._command_to_put_main_system
+
+    def printExecution(self):
+
+        print(Style.BRIGHT + Fore.YELLOW + "ExecutionCommand: System with the API (" + self._command_to_put_main_system.API_name + " along with subsystems):")
+        
+        for c in self.executionCommands:
+            c.printExecution()
+
+        print(Style.BRIGHT + Fore.YELLOW + "}")
+        
+    def codeGen_init(self, language):
+
+        for c in self.executionCommands:
+            c.codeGen_init(language)        
+
+    def codeGen_destruct(self, language):
+        for c in self.executionCommands:
+            c.codeGen_destruct(language)            
+
+    def codeGen(self, language, flag):
+
+        lines = ''
+
+        if language == 'c++':
+
+            if flag == 'variables':
+                pass
+
+            if flag == 'code':
+                #
+        
+
+                lines += '// namespace for ' + self._command_to_put_main_system.API_name + ' {\n'
+
+                # put the global variables
+                innerLines = '// global variables\n'
+
+                for c in self.executionCommands:
+                    innerLines += c.codeGen(language, 'variables')
+
+                innerLines += '\n'
+
+                # put the code
+                for c in self.executionCommands:
+                    innerLines += c.codeGen(language, 'code')
+
+                lines += cgh.indent(innerLines)
+
+                # end namespace (finish)
+                lines += '// end of namespace for ' + self._command_to_put_main_system.API_name + '\n\n'
+
+
+        return lines
+
