@@ -181,7 +181,7 @@ class ExecutionLine():
 
     def __init__(self, signalOrder : List[ Signal ] , dependencySignals : List[ Signal ], dependencySignalsSimulationInputs : List[ Signal ], blocksToUpdateStates : List[ Block ], dependencySignalsThroughStates : List[ Signal ] ):
         self.signalOrder = signalOrder
-        self.dependencySignals = dependencySignals
+        # self.dependencySignals = dependencySignals
         self.dependencySignalsSimulationInputs = dependencySignalsSimulationInputs
         self.blocksToUpdateStates = blocksToUpdateStates
         self.dependencySignalsThroughStates = dependencySignalsThroughStates
@@ -191,8 +191,8 @@ class ExecutionLine():
 
         print(Fore.RED + "dependent sources:")
 
-        for s in self.dependencySignals:
-            print("  - " + s.name )
+        # for s in self.dependencySignals:
+        #     print("  - " + s.name )
 
         print(Fore.RED + "dependent sources (simulation inputs):")
                 
@@ -231,9 +231,9 @@ class ExecutionLine():
 
         # TODO: use sets to merge..
 
-        for s in executionLineToAppend.dependencySignals:
-            if not s in self.dependencySignals and not s in self.signalOrder:
-                self.dependencySignals.append(s)
+        # for s in executionLineToAppend.dependencySignals:
+        #     if not s in self.dependencySignals and not s in self.signalOrder:
+        #         self.dependencySignals.append(s)
 
 
         for s in executionLineToAppend.dependencySignalsSimulationInputs:
@@ -247,6 +247,9 @@ class ExecutionLine():
 
         
 
+
+        original_list_tmp = self.signalOrder.copy()
+
         for s in executionLineToAppend.signalOrder:
             # TODO: (for optimization purposes) 
             # check if there comcon blocks in the list. (only in case a block has more than one
@@ -254,20 +257,30 @@ class ExecutionLine():
             # and another one in self.signalOrder  )
 
             # just append the 
-            self.signalOrder.append( s )
+            if not s in original_list_tmp:
+                self.signalOrder.append( s )
 
-        for s in executionLineToAppend.blocksToUpdateStates:
+            else:
+                print("appendExecutionLine: skipped to add " + s.name)
+
+
+
+
+        original_list_tmp = self.blocksToUpdateStates.copy()
+
+        for b in executionLineToAppend.blocksToUpdateStates:
             # TODO: (for optimization purposes) 
             # check if there comcon blocks in the list. 
 
             # just append the 
-            self.blocksToUpdateStates.append( s )
-
+            if not b in original_list_tmp:
+                self.blocksToUpdateStates.append( b )
 
 
 
 
 
+# NOTE: Simulation inputs might come twiche!
 
 
 
@@ -287,10 +300,11 @@ class BuildExecutionPath:
 
         self.dependencySignalsThroughStates = []
 
-        # the list of reachable signals
-        self.reachableSignals = []
+        # the list of signals to compute in correct order 
+        self.execution_order = []
+        # self.execution_order_without_targets = []
 
-        # For each signgal self.reachableSignals there might be a blocks that
+        # For each signgal self.execution_order there might be a blocks that
         # has an internal memory. It is required to build a list of those blocks
         # that need a state update after their output(s) are calculated.
         self.blocksToUpdateStates = []
@@ -306,16 +320,28 @@ class BuildExecutionPath:
 
 
     def getExecutionLine(self, signalToCalculte : Signal):
-        # get the order of computation steps and their order that have
-        # to be performed to compute 'signalToCalculte'
-        #
-        # For each call to this function, a list is generated that does not contain
-        # signals that are already part of a previous list (that are already computed)
-        #
-        # This function can be called multiple times and returns only the necessaray 
-        # computations. Computations already planned in previous calls of this function
-        # are not listed again. (until resetMarkers() is called)
-        #   
+
+        """
+            get the order of computation steps and their order that have
+            to be performed to compute 'signalToCalculte'
+            
+            For each call to this function, a list is generated that does not contain
+            signals that are already part of a previous list (that are already computed)
+            
+            This function can be called multiple times and returns only the necessaray 
+            computations. Computations already planned in previous calls of this function
+            are not listed again. (until resetMarkers() is called)
+
+            -- results --
+
+            self.execution_order contains the list of signals to comute in the correct order including
+            the target signals. Not included in this list are signals that cross the border to the simulation
+            specified by signalToCalculte.system (coming from an outer system). Further, not included are
+            signals that have been computet in a previous call to getExecutionLine().
+
+            self.dependencySignals contains all signals that are required to comput signalToCalculate
+            and either cross the border of a simulation, 
+        """
 
         # TODO: dependency signals should stay as theiy are but reachableSignals should not contain signals
         #       that already have been calculated. Further, reachableSignals shall also contain dependency if they 
@@ -324,7 +350,7 @@ class BuildExecutionPath:
         print("getExecutionLine on level " + str(self.level) )
 
         # reset the lists
-        self.reachableSignals = []
+        self.execution_order = []
         self.dependencySignals = []
         self.dependencySignalsThroughStates = []
         self.dependencySignalsSimulationInputs = []
@@ -336,13 +362,10 @@ class BuildExecutionPath:
         # compute by traversing the tree
         self.backwardTraverseSignalsExec__(startSignal=signalToCalculte, depthCounter = 0)
 
-        # reverse of the list of reacheable signals gives the order of calculating the individual signals
-        self.reachableSignals.reverse()
-
         # the iteration level
         self.level = self.level + 1
 
-        return ExecutionLine( self.reachableSignals, self.dependencySignals, self.dependencySignalsSimulationInputs, self.blocksToUpdateStates, self.dependencySignalsThroughStates )
+        return ExecutionLine( self.execution_order, self.dependencySignals, self.dependencySignalsSimulationInputs, self.blocksToUpdateStates, self.dependencySignalsThroughStates )
 
     def printExecutionLine(self):
         pass
@@ -356,6 +379,11 @@ class BuildExecutionPath:
         self.markedSignals = []
         self.level = 0
 
+    def place_marker_for_current_level(self, signal):
+        # mark the node/signal as being visited (meaning computed)
+        signal.graphTraversionMarkerMarkVisited(self.level)
+        self.markedSignals.append(signal)
+
     def isSignalAlreadyComputable(self, signal : Signal):
         return signal.graphTraversionMarkerMarkIsVisited()
 
@@ -364,66 +392,28 @@ class BuildExecutionPath:
         
         tabs = ''
         for i in range(0, depthCounter):
-            tabs += '   '
+            tabs += '.  '
 
 
         if not (isinstance(startSignal, SimulationInputSignal) or isinstance(startSignal, BlockOutputSignal)):
             
             # this case must be an error..                  
-            raise BaseException('not implemented or internal error: unexpected type of signal' + startSignal.name)
+            raise BaseException('not implemented or internal error: unexpected type of signal ' + startSignal.name)
 
 
-        if startSignal.graphTraversionMarkerMarkIsVisitedOnLevel(self.level):
-            # startSignal was already computed at this level
 
-
-            # it might be an algeraic loop, but does not have to be
-            #
-            # TODO: investigate if this is able to find all algebraic loops, i.e. also in case
-            #       of multiple levels
-            #
-            #print(tabs + "algebraic loop detected at signal " + startSignal.toStr())
-            #raise BaseException("algebraic loop detected at signal " + startSignal.toStr())
-
-            #
-            # move startSignal to an upper level in the execution linst
-            #
-
-            # remove from its current position
-            self.reachableSignals.remove( startSignal )
-
-            # put to the front
-            self.reachableSignals.append( startSignal )
-
-            print(Style.DIM + tabs + "moved to the current head of the execution list")
-            
-            return
-
-
-        #
-        if startSignal.graphTraversionMarkerMarkIsVisited():
+        if startSignal.graphTraversionMarkerMarkIsVisitedOnLevelLowerThan(self.level):
             # - a previously computed signal has been reached
 
             print(Style.DIM + tabs + "has already been calculated in a previous traversion") 
-
             self.dependencySignals.append( startSignal )
 
-
             return
 
-        # mark the node visited
-        startSignal.graphTraversionMarkerMarkVisited(self.level)
-        self.markedSignals.append(startSignal)
-
-        #
-        # store startSignal as reachable (put it on the exeution list)
-        # NOTE: if startSignal is the tip of the tree (no dependingSignals) it is excluded
-        #       from this list. However, it is still in the list of dependencySignals.
-        #
-
-        print(Style.DIM + tabs + "added " + startSignal.toStr())
-        self.reachableSignals.append( startSignal )
+        if startSignal.graphTraversionMarkerMarkIsVisited():
 
+            print(Style.DIM + tabs + "has already been calculated in this traversion") 
+            return
 
 
 
@@ -435,14 +425,18 @@ class BuildExecutionPath:
             # signal is an input to the simulation
             # add to the list of dependent inputs
 
+            print(Fore.YELLOW + tabs + "  --> crosses system bounds")
+
             # startSignal is at the top of the tree, so add it to the dependiencies
             self.dependencySignals.append( startSignal )
 
             # also note down that this is a (actually used) simulation input
             self.dependencySignalsSimulationInputs.append( startSignal )
 
-            if is_crossing_simulation_border:
-                print(Fore.YELLOW + tabs + "  --> crosses system bounds")
+            print(Style.DIM + tabs + "added input dependency " + startSignal.toStr())
+
+            # mark the node/signal as being visited (meaning computed)
+            self.place_marker_for_current_level(startSignal)
 
             return
 
@@ -451,39 +445,83 @@ class BuildExecutionPath:
         block = startSignal.getSourceBlock()
         blocksPrototype = block.getBlockPrototype()
 
+        #
         # check if the block that yields startSignal uses internal-states to compute startSignal
-        if blocksPrototype.returnInutsToUpdateStates( startSignal ) is not None:
+        #
+
+        inputsToUpdateStatesTmp = blocksPrototype.returnInutsToUpdateStates( startSignal )
+        if inputsToUpdateStatesTmp is not None:
+            print(tabs + "--- signals needed *indirectly* for " + startSignal.name + " (through state update) --" )
+
             # 
             self.blocksToUpdateStates.append( block )
 
-            # add the signals that are required to perform the state update
-            self.dependencySignalsThroughStates.extend( blocksPrototype.returnInutsToUpdateStates( startSignal ) )
+            # please note: blocksPrototype.returnInutsToUpdateStates might return some undetermined signals that are resolved here
+            resolveUndeterminedSignals( inputsToUpdateStatesTmp )
 
+            # add the signals that are required to perform the state update
+            self.dependencySignalsThroughStates.extend( inputsToUpdateStatesTmp )
+
+            for signal in inputsToUpdateStatesTmp:
+                print(Fore.MAGENTA + tabs + "-> S " + signal.name )
+
+            # mark the node/signal as being visited (meaning computed) ?????????????
+            # self.place_marker_for_current_level(startSignal)
+
+        #
         # find out the links to other signals but only these ones that are 
         # needed to calculate 'startSignal'
-        print(tabs + "--- signals needed for " + startSignal.name + " (" + ") --" )
+        #
+        print(tabs + "--- signals needed for " + startSignal.name + " --" )
 
         dependingSignals = blocksPrototype.returnDependingInputs(startSignal)
 
+        # please note: blocksPrototype.returnDependingInputs might return some undetermined signals that are resolved here
+        resolveUndeterminedSignals( dependingSignals )
+
         if len(dependingSignals) == 0:
             # no dependencies to calculate startSignal (e.g. in case of const blocks or blocks without direct feedthrough)
+            print(Style.DIM + tabs + "  (no signals needed) "  )
 
             # block startSignal.getSourceBlock() --> startSignal is a starting point
 
             # startSignal is at the top of the tree, so add it to the dependiencies
             self.dependencySignals.append( startSignal )
 
+            #
+            print(Style.DIM + tabs + "added " + startSignal.toStr())
+            self.execution_order.append( startSignal )
+
+            # mark the node/signal as being visited (meaning computed)
+            self.place_marker_for_current_level(startSignal)
+
             return
 
 
 
+        #
+        # ITERATE: go through all signals needed to calculate startSignal
+        #          only in case there are any, we come to this point
+        #
 
-        # go through all signals needed to calculate startSignal
         for signal in dependingSignals:
 
             print(Fore.MAGENTA + tabs + "-> S " + signal.name )
 
             self.backwardTraverseSignalsExec__( signal, depthCounter = depthCounter + 1 )
 
+        #
+        # FINALIZE: now also startSignal can be computed
+        #
 
+        #
+        # store startSignal as reachable (put it on the exeution list)
+        # NOTE: if startSignal is the tip of the tree (no dependingSignals) it is excluded
+        #       from this list. However, it is still in the list of dependencySignals.
+        #
 
+        print(Style.DIM + tabs + "added " + startSignal.toStr())
+        self.execution_order.append( startSignal )
+
+        # mark the node/signal as being visited (meaning computed)
+        self.place_marker_for_current_level(startSignal)
