@@ -1,5 +1,4 @@
 from . import libdyn
-
 from . import CompileDiagram as cd
 from . import SignalInterface as si
 
@@ -27,9 +26,6 @@ def export_graph(filename, system = None):
     if system is None:
         system = get_simulation_context() 
 
-
-    # filename = 'generated/graph.json'
-
     print()
     print(Style.BRIGHT + "-------- export graph --------")
     print()
@@ -85,3 +81,26 @@ def show_execution_lines(compile_results):
     print()
 
     compile_results.commandToExecute.printExecution()
+
+
+
+
+
+def generate_code(template : PutRuntimeCppHelper, folder=None, build=False):
+
+    # Compile system (propagate datatypes)
+    compile_results = dy.compile_current_system()
+
+    # Build an executable based on a template
+    template.set_compile_results( compile_results )
+    sourcecode, manifest = template.code_gen()
+
+    if folder is not None:
+        # write generated code into a folder and build
+        template.write_code("generated/")
+
+        if build:
+            template.build()
+
+
+    return sourcecode, manifest
